@@ -1,7 +1,7 @@
 /**************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006 2007 2008 2009 2010 2011  Michael Cornelison
+   Copyright 2006 2007 2008 2009 2010 2011 2012  Michael Cornelison
    source URL:  kornelix.squarespace.com
    contact: kornelix2@googlemail.com
    
@@ -20,7 +20,7 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.4.9
+//     zfuncs.h   version  v.5.2
 
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -30,6 +30,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include <math.h>
 #include <stdarg.h>
 #include <fcntl.h>
@@ -38,6 +39,7 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
+#include <gtk/gtk.h>
 
 #define  int8   char                                                       //  number types
 #define  int16  short
@@ -49,7 +51,6 @@
 #define  uint64  unsigned long long
 #define  uchar  unsigned char
 #define  cchar  const char
-#define  dub  double
 
 #define  wstrerror(err) strerror(WEXITSTATUS(err))                         //  get text status for child process
 
@@ -67,9 +68,9 @@
 
 //  system functions ======================================================
 
-void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
 void apppause();                                                           //  output message and wait for user
 void apppause(cchar * pMess, ... );                                        //  same, works like printf
+void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
 void catch_signals();                                                      //  catch signals and do backtrace dump
 void beroot(int argc = 0, char *argv[] = 0);                               //  restart image as root if password OK
 
@@ -84,8 +85,8 @@ double jobtime();                                                          //
 int  parseprocfile(cchar *pfile, cchar *pname, double *value, ...);        //  get data from /proc file
 int  parseprocrec(char *prec, int field, double *value, ...);              //  get data from /proc file record
 void zsleep(double dsecs);                                                 //  sleep specified seconds
-int global_lock(cchar *lockname);                                          //  obtain exclusive lock, multi-process
-int global_unlock(int fd);                                                 //  release the lock
+int global_lock(cchar *lockfile);                                          //  obtain exclusive lock, multi-process
+int global_unlock(int fd, cchar *lockfile);                                //  release the lock
 
 void start_detached_thread(void * tfunc(void *), void * arg);              //  start detached thread function
 void synch_threads(int NT = 0);                                            //  synchronize NT threads
@@ -99,10 +100,10 @@ void gate_threads_leave();                                                 //
 
 char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
 void zfree(void *buff);                                                    //  free() with log option
-void zmalloc_log(int count);                                               //  turn zmalloc() log on/off       v.3.5
-void zmalloc_report();                                                     //  print total memory allocated    v.3.5
+void zmalloc_log(int count);                                               //  turn zmalloc() log on/off
+void zmalloc_report();                                                     //  print memory allocation report
 
-char * command_output(int &contx, cchar *command, ...);                    //  get shell command output        v.2.3
+char * command_output(int &contx, cchar *command, ...);                    //  get shell command output
 int command_status(int contx);                                             //  get exit status of command
 int signalProc(cchar * pname, cchar * signal);                             //  pause/resume/kill subprocess
 int runroot(cchar *sucomm, cchar *command);                                //  run command as root via su or sudo
@@ -112,9 +113,10 @@ int samedirk(cchar *file1, cchar *file2);                                  //  r
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
 
 //  measure CPU time spent in a function or code block within a function
-extern volatile double   cpu_profile_timer;                                //  internal data tables
-extern volatile double   cpu_profile_table[100];
-extern volatile double   cpu_profile_elapsed;
+
+extern volatile double cpu_profile_timer;                                  //  internal data tables
+extern volatile double cpu_profile_table[100];
+extern volatile double cpu_profile_elapsed;
 void cpu_profile_init();                                                   //  initialize at start of test
 void cpu_profile_report();                                                 //  report CPU time per function
 inline void cpu_profile_enter(int fnum)                                    //  at entry to measured code block
@@ -158,7 +160,7 @@ int  blank_null(cchar *string);                                            //  t
 char * strdupz(cchar *string, int more = 0, cchar *tag = 0);               //  duplicate string in nonvolatile memory
 int  strdupz(cchar *source, char *&dest, int more = 0, cchar *tag = 0);    //  same + dest zmalloc if null or too short
 int  clean_escapes(char *string);                                          //  replace \x escapes with real characters
-int  utf8len(cchar *utf8string);                                           //  get graphic cc for UTF8 string   v.2.3
+int  utf8len(cchar *utf8string);                                           //  get graphic cc for UTF8 string
 int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);         //  get graphic substring from UTF8 string
 int  utf8_check(cchar *string);                                            //  check utf8 string for encoding errors
 int  utf8_position(cchar *utf8in, int Nth);                                //  get byte position of Nth graphic char.
@@ -197,13 +199,15 @@ int pvlist_sort(pvlist *pv);                                               //  s
 
 //  number conversion =====================================================
 
-int convSI (cchar *inp, int &inum, cchar **delm = 0);                      //  string to int
-int convSI (cchar *inp, int &inum, int low, int hi, cchar **delm = 0);     //  (with low/high limit checking)
-int convSD (cchar *inp, dub &dnum, cchar **delm = 0);                      //  string to double
-int convSD (cchar *inp, dub &dnum, dub low, dub hi, cchar **delm = 0);     //  (with low/high limit checking)
-int convIS (int iin, char *outp, int *cc = 0);                             //  int to string, returned cc
-int convDS (dub din, int prec, char *outp, int *cc = 0);                   //  double to string, precision, ret. cc
-char * formatKBMB(double fnum, int prec);                                  //  format nnn B, nn.n KB, n.nn MB, etc.
+int convSI (cchar *inp, int &inum, cchar **delm = 0);                            //  string to int
+int convSI (cchar *inp, int &inum, int low, int hi, cchar **delm = 0);           //  (with low/high limit checking)
+int convSD (cchar *inp, double &dnum, cchar **delm = 0);                         //  string to double
+int convSD (cchar *inp, double &dnum, double low, double hi, cchar **delm = 0);  //  (with low/high limit checking)
+int convSF (cchar *inp, float &fnum, cchar **delm = 0);                          //  string to double
+int convSF (cchar *inp, float &fnum, float low, float hi, cchar **delm = 0);     //  (with low/high limit checking)
+int convIS (int iin, char *outp, int *cc = 0);                                   //  int to string, returned cc
+int convDS (double din, int prec, char *outp, int *cc = 0);                      //  double to string, precision, ret. cc
+char * formatKBMB(double fnum, int prec);                                        //  format nnn B, nn.n KB, n.nn MB, etc.
 
 //  wildcard functions ====================================================
 
@@ -212,7 +216,9 @@ cchar * SearchWild(cchar *wpath, int &flag);                               //  w
 
 //  search and sort functions =============================================
 
-int bsearch(int element, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(int seekint, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(char *seekrec, char *allrecs, int recl, int nrecs);            //  binary search sorted records
+int bsearch(char *seekrec, char **allrecs, int N, int nrecs);              //  binary search of sorted pointers to recs
 
 typedef int HeapSortUcomp(cchar * rec1, cchar * rec2);                     //  return -1/0/+1 if rec1 </=/> rec2
 void HeapSort(int vv[], int nn);                                           //  Heap Sort - integer
@@ -231,10 +237,10 @@ int lrandz();                                                              //  b
 double drandz(int64 * seed);                                               //  returns 0.0 to 0.99999...
 double drandz();                                                           //  built-in seed
 
-//  spline curve-fitting functions ========================================  v.2.11
+//  spline curve-fitting functions ========================================
 
-void spline1(int nn, double *dx, double *dy);                              //  define a curve using nn data points
-double spline2(double x);                                                  //  return y-value for given x-value
+void spline1(int nn, float *dx, float *dy);                                //  define a curve using nn data points
+float spline2(float x);                                                    //  return y-value for given x-value
 
 /**************************************************************************/
 
@@ -244,32 +250,26 @@ double spline2(double x);                                                  //  r
 #define ZTXmaxcc 4000                                                      //  max. cc per string
 
 int zinitapp(cchar *appname, ...);                                         //  initz. app directories and files   v.4.1
-cchar * get_zinstalloc();                                                  //  get /usr or /usr/local  ...        v.4.1
+cchar * get_zprefix();                                                     //  get /usr or /usr/local  ...        v.4.1
 cchar * get_zuserdir();                                                    //  get /home/user/.appname/
 cchar * get_zdatadir();                                                    //  get install directory
-cchar * get_zdocdir();                                                     //  get document directory             v.3.8
+cchar * get_zdocdir();                                                     //  get document directory 
 
 void showz_userguide(cchar *context = 0);                                  //  show user guide in new process
 void showz_readme();                                                       //  show README file in popup window
 void showz_changelog();                                                    //  show changelog file in popup window
+void showz_logfile();                                                      //  show log file in popup window      v.5.2
 void showz_translations();                                                 //  show TRANSLATIONS file in popup window
 void showz_doctext(cchar *file);                                           //  show text file (or .gz) in a popup window
 void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
-
-void ZTXinit(cchar *lang);                                                 //  setup for message translation
-cchar *ZTX(cchar *english);                                                //  get translation for English message
-void ZTX_translation_start();                                              //  start online translation
+void showz_html(cchar *url);                                               //  show html via preferred browser
 
 /**************************************************************************
    GTK utility functions
 ***************************************************************************/
 
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-
-#define  zdcbmax 100                                                       //  max. combo box drop-down list
-
 void zmainloop(int skip = 0);                                              //  do main loop, process menu events
+void zthreadcrash();                                                       //  crash if thread is not main() thread
 
 //  text window print and read utilities
 void wprintx(GtkWidget *Win, int line, cchar *mess, cchar *font = 0);      //  write text to line, optional font
@@ -287,7 +287,10 @@ void wprintp(GtkWidget *Win);                                              //  p
 
 //  functions to simplify building menus, tool bars, status bars
 
-#define G_SIGNAL(window,event,func,arg) g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
+#define G_SIGNAL(window,event,func,arg) \
+        g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg)
+
+#define  zdcbmax 100                                                             //  max. combo box drop-down list
 
 typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
 
@@ -299,7 +302,7 @@ GtkWidget * create_popmenu();
 GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname, mtFunc func);     //  add menu item to popup menu
 void popup_menu(GtkWidget *popmenu);                                             //  pop-up menu at current mouse position
 
-GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24, int vert = 0);    //  horiz. toolbar in packing box (vert.)
+GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24);                  //  toolbar in packing box (no vert gtk3)
 GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label, cchar *tip,        //  add button with stock (gtk-quit) 
                                  cchar *icon, mtFunc func);                      //    or custom icon (iconfile.png)
 
@@ -327,6 +330,7 @@ struct zwidget  {
       int         expand;                          //  expandable flag
       int         space;                           //  extra padding space (pixels)
       int         wrap;                            //  wrap mode for edit widget
+      int         stopKB;                          //  stop propagation of KB events to parent
       GtkWidget   *widget;                         //  GTK widget pointer
 };
 
@@ -336,6 +340,7 @@ struct zdialog  {
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
       int         saveposn;                        //  save and recall window position each use
+      int         stopKB;                          //  next KB event will be ignored
       cchar       *help_topic;                     //  optional help topic
       GtkWidget   *parent;                         //  parent window or null
       GtkWidget   *compbutt[zdmaxbutts];           //  dialog completion buttons
@@ -364,12 +369,13 @@ void zdialog_help(zdialog *zd, cchar *help_topic);                         //  s
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
 int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);          //  run dialog, handle events
 
-int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog   v.2.17
-int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status   v.2.23
-int zdialog_destroy(zdialog *zd);                                          //  destroy dialog (caller responsibility)
+int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog
+int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status
+int zdialog_show(zdialog *zd, int flag);                                   //  show or hide a dialog              v.5.2
+int zdialog_destroy(zdialog *zd);                                          //  destroy dialog (caller resp.)
 int zdialog_free(zdialog *&zd);                                            //  free zdialog memory
 int zdialog_wait(zdialog *zd);                                             //  wait for dialog status or be destroyed
-int zdialog_goto(zdialog *zd, cchar *name);                                //  put cursor at named widget  v.2.23
+int zdialog_goto(zdialog *zd, cchar *name);                                //  put cursor at named widget
 void zdialog_set_cursor(zdialog *zd, GdkCursor *cursor);                   //  set cursor for dialog window
 
 int zdialog_stuff(zdialog *zd, cchar *name, cchar *data);                  //  stuff string data into widget
@@ -384,9 +390,9 @@ int zdialog_fetch(zdialog *zd, cchar *name, float &data);                  //  g
 int zdialog_cb_app(zdialog *zd, cchar *name, cchar *data);                 //  append entry to combo drop down list
 int zdialog_cb_prep(zdialog *zd, cchar *name, cchar *data);                //  prepend entry to combo drop down list
 char * zdialog_cb_get(zdialog *zd, cchar *name, int Nth);                  //  get combo drop down list Nth entry
-int zdialog_cb_delete(zdialog *zd, cchar *name, cchar *data);              //  delete combo drop down list entry   v.2.4
-int zdialog_cb_clear(zdialog *zd, cchar *name);                            //  clear all combo box entries         v.2.4
-int zdialog_cb_popup(zdialog *zd, cchar *name);                            //  show all combo box list entries     v.3.8
+int zdialog_cb_delete(zdialog *zd, cchar *name, cchar *data);              //  delete combo drop down list entry
+int zdialog_cb_clear(zdialog *zd, cchar *name);                            //  clear all combo box entries 
+int zdialog_cb_popup(zdialog *zd, cchar *name);                            //  show all combo box list entries
 
 int  zdialog_positions(cchar *action);                                     //  load or save zdialog window positions
 void zdialog_set_position(zdialog *zd, cchar *posn);                       //  set initial/new zdialog window position
@@ -394,12 +400,19 @@ void zdialog_save_position(zdialog *zd);                                   //  s
 
 /**************************************************************************/
 
-//   GTK misc. utility functioins
+//  translation functions
+
+void ZTXinit(cchar *lang);                                                 //  setup for message translation
+cchar *ZTX(cchar *english);                                                //  get translation for English message
+void ZTX_translation_start(GtkWidget *parent);                             //  start online translation
 
 //  write text to popup window, shell command to popup window
+
 int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
 int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);
 
+//  popup message dialogs
+
 void zmessageACK(GtkWidget *parent, cchar *pMess, ... );                   //  display message, wait for OK
 void zmessLogACK(GtkWidget *parent, cchar *pMess, ... );                   //  same, with log to STDOUT
 int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  display message, wait for YES or NO
@@ -408,37 +421,28 @@ zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  d
 char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
 int zdialog_choose(cchar *title, GtkWidget *parent, cchar *mess, ...);     //  show message and return choice button
 
-//  file chooser dialogs for 1 file or multiple files
+//  file chooser dialogs for one file or multiple files
+
 char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 char ** zgetfileN(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 
-void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
+//  print image file
 
-typedef void drag_drop_func(int x, int y, char *text);                     //  user function, get drag_drop text  v.2.19
-void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  connect window to user function
-
-GdkCursor * zmakecursor(cchar *iconfile);                                  //  make a cursor from an image file   v.3.7
-
-/**************************************************************************/
-
-//   GDK/GTK image file utility functions
-//   for functions returning char *, caller responsible for zfree()
-
-void print_image_paper_setup();                                            //  paper setup for printing an image file
-void print_image_margins_setup();                                          //  setup print margins
+void print_image_paper_setup(GtkWidget *parent);                           //  paper setup for printing an image file
+void print_image_margins_setup(GtkWidget *parent);                         //  setup print margins
 void print_image_file(cchar *imagefile);                                   //  print the image file
 
-GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
+//  drag and drop functions
 
-char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  display image gallery window, navigate, 
-       void ufunc(int Nth, int butt) = 0, GtkWidget *parent = 0);          //    do callback for clicked thumbnails
+typedef void drag_drop_func(int x, int y, char *text);                     //  user function, get drag_drop text
+void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  connect window to user function
 
-int image_gallery_position(cchar *file, int Nth);                          //  get relative position of file in gallery
-int image_file_type(cchar *file);                                          //  determine if directory or image file type
-char * image_thumbfile(char *imagefile);                                   //  get thumbnail filespec, create if missing
-GdkPixbuf * image_thumbnail(char *imagefile, int size = 0);                //  get thumbnail pixbuf, create if missing
+//  miscellaneous GDK/GTK functions
 
-char ** image_gallery_getfiles(char *startdir, GtkWidget *parent = 0);     //  select files from gallery window
+GdkPixbuf * get_thumbnail(char *fpath, int size);                          //  get sized thumbnail for image file
+GdkCursor * zmakecursor(cchar *iconfile);                                  //  make a cursor from an image file
+int gdk_window_move_pointer(GdkWindow *, int px, int py);                  //  move the mouse pointer to px, py
+GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, float deg, int alfa = 0);       //  rotate pixbuf through any angle
 
 /**************************************************************************/
 
